import { BehaviorSubject, Subject, buffer, bufferWhen, elementAt, filter, interval } from 'rxjs';
import { DataPrepService } from '../services/dataprep.service';
import { StorageLocation } from '../types/interface';
import { Server } from 'socket.io'
import { io } from 'socket.io-client';

let msgData = new DataPrepService()

/* ---------------------- COMPLEX OPERATION ------------------------------ */
let msgPayload: Subject<any> = new Subject();
let consumerTrafficStatus: Subject<any> = new Subject()
let bufferTrigger: BehaviorSubject<boolean> = new BehaviorSubject(true)

let mongoStorage: StorageLocation = {
    type: `MongoDB`,
    url: `mongodb://192.168.100.59:27017/default`
}
msgData.loadObsData(mongoStorage, msgPayload)

consumerTrafficStatus.subscribe((element) => {
    if (element >= 2) {
        let warning: boolean = false
        bufferTrigger.next(warning)
        console.log(`Heap Load Exceeded on client side. Buffering.....`)
    } else {
        let warning: boolean = true
        bufferTrigger.next(warning)
        // console.log(`Releasing the buffer`)
    }
})


// Create a WebSocket server
function createWebsocketServer() {
    const io = new Server({})
    io.on('connection', (socket) => {
        console.log(`Consumer Connected`)

        // Subscribe to the subject when a client connects
        const subscription = msgPayload.pipe(
            buffer(bufferTrigger.pipe(filter(Boolean)))
        ).subscribe((element) => {
            console.log(`Emitting ${element.length} messages`)
            // let messageString = JSON.stringify(element)
            socket.emit(`payload`, element);
        });

        //Listen for messages from consumer/client
        socket.on(`message`, (message) => {
            console.log(`Received message from client: ${message}`)

            // Send a message back to the client
            socket.send(`${message} received!`);
        })

        // Listen for the socket to be closed
        socket.on('disconnect', () => {
            console.log('Client disconnected');
            subscription.unsubscribe();
            consumerTrafficStatus.next(false)
        });
    })

    io.listen(8080)
}


// Create a new WebSocket client
function connectWebSocket() {
    const socket = io('http://localhost:8081');

    socket.on('connect', () => {
        console.log('Connected to publisher');
    });

    socket.on('trafficControl', (report) => {
        consumerTrafficStatus.next(report);
    });

    socket.on('disconnect', () => {
        console.log('Disconnected from server');

        // Attempt to reconnect every 3 seconds
        setTimeout(() => {
            console.log('Attempting to reconnect...');
            socket.connect();
        }, 3000);
    });
}

createWebsocketServer()
connectWebSocket();
